/*
 * Created on 08.07.2005
 *
 * TOODO JL: more testcases for the MiniBrowser
 */
package isac.gui.browser.minibrowser;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.net.URL;

import javax.swing.event.HyperlinkEvent;

import isac.gui.browser.minibrowser.MiniBrowser;
import junit.framework.TestCase;

/**
 * MiniBrowser
 * 
 * This testcases should test the MiniBrowser independend from the
 * implementation. So only functional use cases are tested.
 * 
 * All tests have to be done after each other (synchronized). The MiniBrowser
 * throws an event if the browser contend is updated. This is signalized with
 * the page_update_event_ flag. Then the assertion can be done. Therefore this
 * class is implemented as event listender.
 * 
 * @author JL
 * @version $Revision$
 * @see isac.gui.browser.minibrowser.IMiniBrowser
 * @see isac.gui.browser.minibrowser.MiniBrowser
 *  
 */
public class TestMiniBrowser extends TestCase implements PropertyChangeListener {

	private boolean page_update_event_ = false;
	
	/*
	 * testNewMiniBrowserWithLocalPage
	 * 
	 * Tests the generation of a MiniBrowser and the representation of a local
	 * (isac internal) page (problem, excample, ...)
	 * 
	 * @author JL
	 */
	public synchronized void testNewMiniBrowserWithLocalPage() throws IOException {
		
		System.out
				.println("---BEGIN isac.functest.minibrowser.testNewMiniBrowserWithLocalPage");

		ProxyMock proxy_mock = new ProxyMock();
		MiniBrowser mini_browser = new MiniBrowser(proxy_mock);

		this.page_update_event_ = false;
		mini_browser
//				.setPage("file:bin/java/isac/functest/minibrowser/test.html");
				.setPage("file:isac/src/java-tests/isac/gui/browser/minibrowser/test.html");
		mini_browser.addPropertyChangeListener(this);
		while(!this.page_update_event_);

		String result = mini_browser.getText();
		String result_first_line = result.substring(0, result.indexOf("\n"));
		String expected_first_line = "<html math=\"http://www.w3.org/1998/Math/MathML\" xlink=\"http://www.w3.org/1999/xlink\" xmlns=\"#DEFAULT\">";
		
		assertEquals(expected_first_line, result_first_line);
		assertEquals("isac/src/java-tests/isac/gui/browser/minibrowser/test.html",
				mini_browser.getPage().getFile());

		System.out
				.println("---END isac.functest.minibrowser.testNewMiniBrowserWithLocalPage");
	}

	/*
	 * testKlickOnLink
	 * 
	 * Tests if the right event is fired
	 * 
	 */
	public synchronized void testKlickOnLink() throws IOException {
		
		System.out
				.println("---BEGIN isac.functest.minibrowser.testKlickOnLink");

		ProxyMock proxy_mock = new ProxyMock();
		MiniBrowser mini_browser = new MiniBrowser(proxy_mock);
		mini_browser.addPropertyChangeListener(this);

		this.page_update_event_ = false;
		mini_browser
				.setPage("file:isac/src/java-tests/isac/gui/browser/minibrowser/test.html");
		while(!this.page_update_event_);

		this.page_update_event_ = false;
		URL url = new URL("file:isac/src/java-tests/isac/gui/browser/minibrowser/test2.html");
		HyperlinkEvent hyperlink_event = new HyperlinkEvent(mini_browser,
				HyperlinkEvent.EventType.ACTIVATED, url);
		
		mini_browser.fireHyperlinkUpdate(hyperlink_event);
		while(!this.page_update_event_);

		assertEquals("No Event fired.", proxy_mock.hyperlink_event_fired_, true);

		String expected_string = "<html>\n  <head>\n    \n  </head>\n  <body>\n    Test Page 2\n  </body>\n</html>\n";
		assertEquals(expected_string, mini_browser.getText());

		System.out.println("---END isac.functest.minibrowser.testKlickOnLink");
	}

	/* 
	 * Catches the PropertyChangeEvent of the MiniBrowser.
	 * 
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 * 
	 * @param property_change_event the Event to be catched
	 */
	public void propertyChange(PropertyChangeEvent property_change_event) {
		if(property_change_event.getPropertyName().equals("page")) {
			this.page_update_event_ = true;
		}
	}

}