/*
 * Created on 26.07.2005
 *
 */
package isac.browserdialog;

import isac.interfaces.IToGUI;
import isac.session.ISessionManager;
import isac.session.SessionManager;
import isac.useractions.IUIAction;
import isac.useractions.IUIElement;
import isac.useractions.IUserAction;
import isac.useractions.UIAction;
import isac.useractions.UIActionNewProblemForBrowser;
import isac.useractions.UserActionOnKEStoreID;
import isac.useractions.UserActionOnKEStoreIDAndWSDID;
import isac.useractions.UserActionOnString;
import isac.util.formulae.KEStoreID;
import isac.util.formulae.Match;
import isac.util.formulae.ProblemID;
import isac.wsdialog.DialogProtocolException;
import isac.wsdialog.IBrowserDialogToWorksheetDialog;
import isac.wsdialog.IWorksheetDialog;

import java.rmi.RemoteException;

/**
 * The browser dialog for the problem browser
 * 
 * @author JL
 * 
 * TOOODO JL implement ProblemBrowserDialog
 */
public class ProblemBrowserDialog extends BrowserDialog {

    private boolean auto_match_mode_;

    private boolean browser_ready_ = false;

    private ProblemID selected_id_;

    private String active_worksheet_id_;

    /**
     * @param session_id
     * @param session
     * @throws RemoteException
     */
    public ProblemBrowserDialog(String session_id, ISessionManager session)
            throws RemoteException {
        super(session_id, session);

        auto_match_mode_ = true;
    }

    /*
     * (non-Javadoc)
     * 
     * @see isac.browserdialog.BDialog#notifyUserAction(isac.wsdialog.IUserAction)
     */
    public boolean notifyUserAction(IUserAction action) throws RemoteException,
            DialogProtocolException {
        System.out.println("ProblemBrowserDialog::notifyUserAction()");
        switch (action.getActionID()) {
        case (IUserAction.UI_PROBLEMBROWSER_PROBLEM_TO_WORKSHEET): {
            return doProblemToWorksheet();
        }
        case (IUserAction.UI_PROBLEMBROWSER_AUTO_MATCH_ON): {
            return setToAutoMatchModeOn();
        }
        case (IUserAction.UI_PROBLEMBROWSER_AUTO_MATCH_OFF): {
            return setToAutoMatchModeOff();
        }
        case (IUserAction.UI_PROBLEMBROWSER_REFINE): {
            return startRefinement();
        }
        case (IUserAction.UI_PROBLEMBROWSER_SELECTION_CHANGED): {
        	ProblemID problem_id = (ProblemID)((UserActionOnKEStoreIDAndWSDID)action).getKEStoreID();
        	String wsd_id = ((UserActionOnKEStoreIDAndWSDID)action).getWSDID();
            if (auto_match_mode_)
                return doTryMatch(problem_id, wsd_id) 
						&& selectedProblemChanged(problem_id);
            else
                return selectedProblemChanged(problem_id);
        }
        case (IUIElement.UI_PROBLEMBROWSER_SELECTED_ID): {
            selected_id_ = (ProblemID) ((UserActionOnKEStoreID) action)
                    .getObjectID();
            browser_ready_ = true;
            return true;
        }
        case (IUIElement.UI_PROBLEMBROWSER_ACTIVE_WORKSHEET): {
            active_worksheet_id_ = ((UserActionOnString) action).getString();
            browser_ready_ = true;
            return true;
        }
        }

        return true;
    }

    /**
     * starts the try match procedure
     * @param wsd_id
     * @param kestore_id
     * 
     * @return true if no exception is thrown
     */
    private boolean doTryMatch(ProblemID kestore_id, String wsd_id) {
        System.out.println("ProblemBrowserDialog::doTryMatch()");
        try {

        	SessionManager session_manager = SessionManager.getInstance();
        	IBrowserDialogToWorksheetDialog worksheet_dialog = (IBrowserDialogToWorksheetDialog) session_manager.getWorksheetDialog(wsd_id);
        	Match match = worksheet_dialog.tryMatchProblem(kestore_id);
        	if(match == null)
        		System.out.println("ProblemBrowserDialog::doTryMatch() NULL");
        	else
        		System.out.println("ProblemBrowserDialog::doTrymatch() not null");
        	
        } catch (Exception e) {
            e.printStackTrace();
        }

        return true;
    }

    /**
     * Starts the refinement procedure
     * 
     * @TOOODO JL: implement
     * @return
     */
    private boolean startRefinement() {
        return false;
    }

    /**
     * loads the new content into the mini browser of the new selected problem
     * @param kestore_id
     */
    private boolean selectedProblemChanged(KEStoreID kestore_id) {
        try {
            //TOOODO JL: sending back a new problem for the browser
            browser_frame_rmi_.doUIAction(new UIActionNewProblemForBrowser(
                    IUIElement.UI_PROBLEMBROWSER_SELECTION_CHANGED,
                    UIAction.UI_CONTEXT_PROBLEM_BROWSER_PANEL, "link"));
        } catch (RemoteException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return false;
    }

    /**
     * Does NOT realy open the problem browser, but does add all buttons after
     * opening the problem broser. This is necessarry because the opening of the
     * browser is not done dynamically. But the dialog has to notice that the
     * browser is obened, so it can add the buttons.
     * 
     * @return
     */
    private boolean openProblemBrowser() {
        if (auto_match_mode_ == true) {
            generateButtonForAutoMatchOff();
            generateButtonForCopyToWorksheet();
            generateButtonForTryRefine();
        } else {
            generateButtonForAutoMatchOn();
        }
        return true;
    }

    /**
     * to set the auto match mode off
     * 
     * @return
     */
    private boolean setToAutoMatchModeOff() {
        if (auto_match_mode_ == true) {
            removeButtonForAutoMatchOff();
            removeButtonForCopyToWorksheet();
            removeButtonForTryRefine();
            generateButtonForAutoMatchOn();
            auto_match_mode_ = false;
            return true;
        }
        return false;
    }

    /**
     * removes the auto match off button
     */
    private void removeButtonForAutoMatchOff() {
        try {
            browser_frame_rmi_.removeUIElement(new UIAction(
                    IUIElement.UI_PROBLEMBROWSER_AUTO_MATCH_OFF,
                    IUIAction.UI_CONTEXT_PROBLEM_BROWSER));
        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }

    /**
     * removes the try refine button
     */
    private void removeButtonForTryRefine() {
        try {
            browser_frame_rmi_.removeUIElement(new UIAction(
                    IUIElement.UI_PROBLEMBROWSER_REFINE,
                    IUIAction.UI_CONTEXT_PROBLEM_BROWSER));
        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }

    /**
     * removes the copy to worksheet button
     */
    private void removeButtonForCopyToWorksheet() {
        try {
            browser_frame_rmi_.removeUIElement(new UIAction(
                    IUIElement.UI_PROBLEMBROWSER_PROBLEM_TO_WORKSHEET,
                    IUIAction.UI_CONTEXT_PROBLEM_BROWSER));
        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }

    /**
     * removes the button for the auto match = on
     */
    private void removeButtonForAutoMatchOn() {
        try {
            browser_frame_rmi_.removeUIElement(new UIAction(
                    IUIElement.UI_PROBLEMBROWSER_AUTO_MATCH_ON,
                    IUIAction.UI_CONTEXT_PROBLEM_BROWSER));
        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }

    /**
     * Sets the browserdialog to the auto match mode and generates the
     * neccesarry buttons
     *  
     */
    private boolean setToAutoMatchModeOn() {
        if (auto_match_mode_ == false) {
            removeButtonForAutoMatchOn();
            generateButtonForAutoMatchOff();
            generateButtonForCopyToWorksheet();
            generateButtonForTryRefine();
            auto_match_mode_ = true;
            return true;
        } else
            return false;
    }

    /**
     * to transfer the selected problem to the worksheet
     * 
     * @return
     */
    private boolean doProblemToWorksheet() {
        // TODO Auto-generated method stub
        return false;
    }

    /**
     * deletes the "auto match off" button and generates the "auto match on"
     * button
     *  
     */
    private void generateButtonForAutoMatchOff() {
        try {
            browser_frame_rmi_.addUIElement(new UIAction(
                    IUIElement.UI_PROBLEMBROWSER_AUTO_MATCH_OFF,
                    IUIAction.UI_CONTEXT_PROBLEM_BROWSER));
        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }

    /**
     * deletes all button and generates the "auto match off button"
     *  
     */
    private void generateButtonForAutoMatchOn() {
        try {
            browser_frame_rmi_.addUIElement(new UIAction(
                    IUIElement.UI_PROBLEMBROWSER_AUTO_MATCH_ON,
                    IUIAction.UI_CONTEXT_PROBLEM_BROWSER));
        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }

    /**
     * generates the button for the try refine
     *  
     */
    private void generateButtonForTryRefine() {
        try {
            browser_frame_rmi_.addUIElement(new UIAction(
                    IUIElement.UI_PROBLEMBROWSER_REFINE,
                    IUIAction.UI_CONTEXT_PROBLEM_BROWSER));
        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }

    /**
     * generates the button for the "copy to worksheet"
     *  
     */
    private void generateButtonForCopyToWorksheet() {
        try {
            browser_frame_rmi_.addUIElement(new UIAction(
                    IUIElement.UI_PROBLEMBROWSER_PROBLEM_TO_WORKSHEET,
                    IUIAction.UI_CONTEXT_PROBLEM_BROWSER));
        } catch (RemoteException remote_exception) {
            remote_exception.printStackTrace();
        }
    }

    /**
     * registeres the BrowserFrame to this dialog
     * 
     * @param browser_frame
     *            the browser frame to register
     */
    public void registerBrowserFrame(IToGUI browser_frame)
            throws RemoteException {
        browser_frame_rmi_ = browser_frame;
        openProblemBrowser();
    }

}