/****************************************************************** 
 * Copyright (c) 2003, Mario Hochreiter, Student of 
 * Software Engineering at FH-Hagenberg, Austria. 
 * Project member of the isac team at the Institute for 
 * Software-Technologie, Graz University of Technology, Austria. 
 * 
 * Use is subject to license terms.
 ******************************************************************/

package isac.gui.browser.panels;

import isac.gui.util.Hierarchy;

import java.awt.BorderLayout;
import java.awt.Dimension;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

/**
 * Panel in the window application that is responsible to layout the browser
 * windows (e.g. example browser). The different browser windows are shown as
 * different tabs in a JTabbedPane.
 * 
 * @author Mario Hochreiter, JL
 * @version 0.1
 *  
 */
public class BrowserPanel extends JPanel {

    /** Contains all "browser" panels like "Example browser". */
    private JTabbedPane browser_tab_pane_;

    /** The mother frame */
    private JFrame root_;

    public BrowserPanel(JFrame root) {
        this.root_ = root;
        this.setMinimumSize(new Dimension(200, 200));
        browser_tab_pane_ = new JTabbedPane(JTabbedPane.BOTTOM,
                JTabbedPane.SCROLL_TAB_LAYOUT);
        this.setLayout(new BorderLayout());
        this.add(browser_tab_pane_, BorderLayout.CENTER);
    }

    public void initExamplesBrowser(Hierarchy hierarchy) {
        browser_tab_pane_.addTab("Example Browser", new ExampleBrowserPanel(
                root_, hierarchy));
    }

    public void initProblemBrowser(Hierarchy hierarchy) {
        browser_tab_pane_.addTab("Problem Browser", new ProblemBrowserPanel(
                root_, hierarchy));
    }

    public void initMethodBrowser(Hierarchy hierarchy) {
        browser_tab_pane_.addTab("Method Browser", new MethodBrowserPanel(
                root_, hierarchy));
        browser_tab_pane_.addTab("Theory Browser", new JPanel());
    }
}