/**
 * Handles all the sessions of the multi-user system.
 * is a singleton; 
 * each session has 1 {isac.browserdialog.BrowserDialog} and
 * 0..n {isac.wwdialog.WorksheetDialog}
 * 
 * TODO.WN050712 rename to SessionHandler
 * 
 * @author Andreas Griesmayer
 * Created on Jun 24, 2003
 */
package isac.session;

import isac.browserdialog.BDialog;
import isac.browserdialog.BrowserDialog;
import isac.browserdialog.ProblemBrowserDialog;
import isac.util.users.Accounts;
import isac.util.users.User;
import isac.wsdialog.IWorksheetDialog;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.HashMap;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.apache.xmlrpc.WebServer;

/**
 * @author Andreas Griesmayer
 */
public class SessionManager extends UnicastRemoteObject implements
        ISessionManager, Serializable {

    static Logger logger_ = Logger.getLogger(SessionManager.class.getName());

    private int counter_ = 0;

    private static WSDHandler dg_handler_ = new WSDHandler();

    private Map username_map_sessionid_ = new HashMap();

    //private static UMHandler um_handler_ = new UMHandler();WN050725
    private Map sessionid_map_user_ = new HashMap();

    //actually holds (1) null from loadHierarchy (2) "x0" or "
    private Map bdialogmap_ = new HashMap();

    /**
     * Map for all problem browser dialogs of all sessions
     */
    private Map problem_browser_dialog_map_ = new HashMap();

    /**
     * Map for all example browser dialogs of all sessions
     */
    private Map example_browser_dialog_map_ = new HashMap();

    /**
     * Map for all method browser dialogs of all sessions
     */
    private Map method_browser_dialog_map_ = new HashMap();

    private static SessionManager instance_;

    private SessionManager() throws RemoteException {
        init();//WN050704 called twice on startup: loadHierarchy + login?
        System.out.println("SessionManager constructor");
    }

    /**
     * returns the instance of the singleton SessionManager. The SessionManager
     * is responsible for maintaining all session-relevant informations like
     * Browser- and WorksheetDialog and UserModel it provides its functions
     * using RMI and XML-RPC
     * 
     * @return
     */
    public static SessionManager getInstance() {
        if (instance_ == null) {
            try {
                instance_ = new SessionManager();
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
        return instance_;
    }

    // LK 04.09.02: user control with the UserManager
    /**
     * @param username
     *            name of the user
     * @param pass
     *            password
     * @return vector with the sessionid as first and only element on success or
     *         empty vector on failure
     */
    public Vector login(String username, String pass) {
        Vector return_val = new Vector();
        String sessionid = null;
        //-----------------------------------------------------------NEW
        //TOOODO.WN050311 get user via UserHandler !!!
        String userpath = "isac/xmldata/users/";//TOOODO.WN050311 fix path
        Accounts acs = new Accounts(userpath);//get user via UserHandler
        if (acs.exists(username)
                && acs.getProperty(username).compareTo(pass) == 0) {
            if ((sessionid = (String) username_map_sessionid_.get(username)) == null) {
                {
                    sessionid = username + counter_++;
                    username_map_sessionid_.put(username, sessionid);
                }
                sessionid_map_user_
                        .put(sessionid, new User(username, userpath));
                //TODO get user via UserHandler .......^^^^
            }
            return_val.add(sessionid);
        }
        logger_.fatal("WA<-SM: login <- sessionid=" + sessionid);
        return return_val;
    }

    /**
     * @param session_id_
     * @return the user for the session id
     */
    public User getUser(String sessionId) {
        return (User) sessionid_map_user_.get(sessionId);
    }

    /**
     * returns the name of the user which owns the given session.
     * 
     * @param session
     *            ID of the session for which the owner is requested
     * @return the name of the owner of the session TODO.WN050311 encapsulate
     *         User#getName !!!
     */
    public String getUsername(String session) {
        return ((User) sessionid_map_user_.get(session)).getUsername();
    }

    /**
     * create a new <code>WorksheetDialog</code> and assign it to session
     * <code>session</code>
     * 
     * FIXME: rename to openWorksheetDialog
     */
    public String openWorksheetDialog(String session) {
        return dg_handler_.newWorksheetDialog(this, session);
    }

    public IWorksheetDialog getWorksheetDialog(String dg_id) {
        return (IWorksheetDialog) dg_handler_.getWorksheetDialog(dg_id);
    }

    /**
     * used if the SessionManager is used in standalone mode for
     * testing-purposes. Registers its XML-RPC services on port 1050 of the
     * local machine
     * 
     * @param args
     *            not used
     */
    public static void main(String[] args) {
        init();
    }

    private static void init() {
        try {
            System.out.println("SessionManager main");

        } catch (Exception exception) {
            System.err.println("SessionManager: " + exception.toString());
        }

    }

    public void finalize() {
        System.out.println("SessionManager finalize!");
    }

    /**
     * returns the BrowserDialog identified by <code>bdialog_id</code> or
     * creates a new one.
     * 
     * 
     * @param bdialog_id
     *            ID of the BrowserDialog to open.
     * 
     * @return RMI-Object of the requested BrowserDialog
     * 
     * @deprecated should be no longer used because the dialogs for problem,
     *             example, and methods are stored in seperate maps
     */
    public BDialog getBDialog(String session) {
        BDialog browser_dialog = (BDialog) bdialogmap_.get(session);
        if (browser_dialog == null) {
            try {
                bdialogmap_.put(session, (browser_dialog = new BrowserDialog(
                        session, this)));
            } catch (RemoteException e) {
                System.err
                        .println("unable to create new BrowserDialog for session "
                                + session);
                e.printStackTrace();
            }
        }
        return browser_dialog;
    }

    /**
     * Generates a new ProblemBrowserDialog or returns the existing one
     * 
     * @param session_id_
     * @return new or existing prolblem browser dialog (if exists)
     */
    public BDialog getProblemBrowserDialog(String session_id)
            throws RemoteException {
        BDialog browser_dialog = (BDialog) problem_browser_dialog_map_
                .get(session_id);
        User user = getUser(session_id);
        if (browser_dialog == null) {
            try {
                problem_browser_dialog_map_.put(session_id,
                        (browser_dialog = new ProblemBrowserDialog(session_id,
                                this)));
            } catch (RemoteException e) {
                System.err
                        .println("unable to create new ProblemBrowserDialog for session "
                                + session_id);
                e.printStackTrace();
            }
        }
        return browser_dialog;
    }
}