/*
 * @author Alois Kirchsteigerr, Alan Krempler
 * created Jul 11, 2005 11:13:42 AM
 * Institute for Softwaretechnology, Graz University of Technology, Austria.
 * 
 * Use is subject to PGPL license terms.
 */
package isac.useractions;

import java.io.Serializable;

/**
 * A context provides for grouping of interactive elements on the gui. The
 * grouping concerns the data to be sent with <code>IUIAction</code>
 * (DialogIterator, CalcElement, etc.) as well as the location of the elements
 * (on the menu-bar, in the context-menu, etc.)
 * 
 * @see IUserAction for reverse direction (i.e. GUI --> Dialog)
 * 
 * @author Alois Kirchsteigerr, Alan Krempler
 */
public interface IUIAction extends IUIElement, Serializable {

    /** the context for a whole calculation (0507 on the mainframe) */
    public static final int UI_CONTEXT_CALCULATION = 1;

    /** for all calcelements together existing in a calculation (0507 context */
    public static final int UI_CONTEXT_CALCELEMENTS = 2;

    /** for one specific element in a calculation (0507 added to context menu) */
    public static final int UI_CONTEXT_ONEELEMENT = 3;

    /** for the whole ProblemBrowser */
    public static final int UI_CONTEXT_PROBLEM_BROWSER = 4;

    /** for the panel in the problem browser **/
    public static final int UI_CONTEXT_PROBLEM_BROWSER_PANEL = 5;

    /** for the minibrowser in the problem browser **/
    public static final int UI_CONTEXT_PROBLEM_BROWSER_MINIBROWSER = 6;

    /**
     * not a useraction, but for technical reasons: for lauching and releasing
     * the CalcHeadPanel
     */
    public static final int UI_CONTEXT_SWITCH_PHASES = 7;

    /** for all elements on the CalcHeadPanel */
    public static final int UI_CONTEXT_CALCHEAD = 8;

    /**
     * @return An integer value describing the context of the UIElement
     */
    public int getContext();

    public int getAction();

    public String getLabel();

}