/*
 * @author Alan Krempler
 * Created on Dec 12, 2003
 */
package isac.useractions;

/**
 * @author Alan Krempler
 * 
 * Constants describing interactions or request from the user TOOODO.AK? rename
 * to IUInteractions ???
 */
public interface IUIElement {

    /** ***************** NOTIFICATIONS: SPECIFYING PHASE ********************** */

    public static final int UI_DUMMY_FIRST_SPECIFY = 1;

    /**
     * During the specifying phase, change the view on the CalcHead. WN050804
     * how to use ? which view ?
     */
    public static final int UI_SPECIFY_CHANGE_VIEW = 3;

    /**
     * During the specifying phase, have the Model of the problem (or the guard
     * of the method) completed by the math engine.
     */
    public static final int UI_SPECIFY_COMPLETE_MODEL = 4;

    /**
     * During the specifying phase, have the CalcHead completed by the math
     * engine.
     */
    public static final int UI_SPECIFY_COMPLETE_CALCHEAD = 5;

    /**
     * During the specifying phase, have the Method field of the CalcHead
     * completed by the math engine.
     */
    public static final int UI_SPECIFY_METHOD = 6;

    /**
     * During the specifying phase, have the Theory field of the CalcHead
     * completed by the math engine.
     */
    public static final int UI_SPECIFY_THEORY = 7;

    /**
     * During the specifying phase, have the Problem field of the CalcHead
     * completed by the math engine.
     */
    public static final int UI_SPECIFY_PROBLEM = 8;

    /**
     * During the specifying phase, have the Given field of the CalcHead
     * completed by the math engine.
     */
    public static final int UI_SPECIFY_COMPLETE_GIVEN = 9;

    /**
     * During the specifying phase, have the Find field of the CalcHead
     * completed by the math engine.
     */
    public static final int UI_SPECIFY_COMPLETE_FIND = 10;

    /**
     * During the specifying phase, have the Relate field of the CalcHead
     * completed by the math engine.
     */
    public static final int UI_SPECIFY_COMPLETE_RELATE = 11;

    /**
     * During the specifying phase, have the CalcHead checked for completeness
     * and consistency.
     */
    public static final int UI_SPECIFY_CHECK_CALCHEAD = 12;

    /**
     * Leave the specifying phase, enter the solving phase and calculate until a
     * final result is reached.
     */
    public static final int UI_SPECIFY_CALCULATE_ALL = 13;

    /**
     * clear the (possibly messed) contents of the CalcHeadPanel such that a
     * UI_SPECIFY_COMPLETE_CALCHEAD or a UI_SPECIFY_CALCULATE_ALL can succeed.
     */
    public static final int UI_SPECIFY_RESET = 14;

    /**
     * open the CalcHeadPanel and start the specify-phase, if the CalcHead is
     * not 'complete'
     */
    public static final int UI_SPECIFY_OPEN_CALCHEAD = 15;

    /**
     * Leave the specifying phase, enter the solving phase and calculate the
     * first step of the method. A UserAction only.
     * 
     * @see UI_SPECIFY_CLOSE_CALCHEAD the closely related UIAction
     */
    public static final int UI_SPECIFY_TO_SOLVE = 16;

    /**
     * close the CalcHeadPanel (i.e. a UIAction only)
     * 
     * @see UI_SPECIFY_TO_SOLVE the closely related UserAction
     */
    public static final int UI_SPECIFY_CLOSE_CALCHEAD = 17;

    /**
     * fill calchead elementwise
     */
    public static final int UI_SPECIFY_COMPLETE_STEPWISE = 18;

    /**
     * Show the guard of the method (already specified in the CalcHead). The
     * guard of the method has the same structure as the Model of a problem;
     * thus the guard is inserted into the CalcHead at the place of the problems
     * Model.
     * 
     * @see UI_SPECIFY_SWITCH_TO_MODEL switch to the other part of a
     *      toggle-mode.
     */
    public static final int UI_SPECIFY_SWITCH_TO_GUARD = 19;

    /**
     * Switch back from having the guard of a method to having the Model of the
     * problem (already specified in the CalcHead).
     * 
     * @see UI_SPECIFY_SWITCH_TO_GUARD switch to the other part of a
     *      toggle-mode.
     */
    public static final int UI_SPECIFY_SWITCH_TO_MODEL = 20;
    
    public static final int UI_SPECIFY_FOCUS_LOST = 21;

    public static final int UI_DUMMY_LAST_SPECIFY = 22;

    /** ***************** NOTIFICATIONS: SOLVING PHASE ********************** */

    public static final int UI_DUMMY_FIRST_SOLVE = 121;

    /**
     * During the solving phase, calculate one more step.
     */
    public static final int UI_SOLVE_CALCULATE_1 = 121;

    /**
     * During the solving phase, calculate until a final result is reached.
     */
    public static final int UI_SOLVE_CALCULATE_ALL = 122;

    /**
     * During the solving phase, calculate until the current subproblem is
     * solved.
     */
    public static final int UI_SOLVE_CALCULATE_SUBPROBLEM = 123;

    /**
     * During the solving phase, request editing the currently active formula.
     * 
     * WN0412 If request is conceded, do
     * 
     * @see UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE
     */
    public static final int UI_SOLVE_EDIT_ACTIVE_FORMULA = 124;

    /**
     * During the solving phase, notify that editing the currently active
     * formula is finished. This implies a request for updating the CalcTree.
     */
    public static final int UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE = 125;

    /**
     * During the solving phase, request inserting a new formula _after_ the
     * currently active formula, making the new formula the currently active
     * formula. At present, this does not imply editing the formula and must be
     * requested in a separate step. WN0412: ...??? is this right interpreted by
     * ... WN0412 If request is conceded, do
     * 
     * @see UI_SOLVE_EDIT_ACTIVE_FORMULA_COMPLETE
     */
    public static final int UI_SOLVE_APPEND_USER_FORMULA = 126;

    /**
     * During the solving phase, request making the referened formula the
     * currently active formula.
     */
    public static final int UI_SOLVE_MOVE_ACTIVE_FORMULA = 127;

    /**
     * During the solving phase, ask for the tactic the math engine would apply
     * to the current formula.
     */
    public static final int UI_SOLVE_GET_PROPOSED_TACTIC = 130;

    /**
     * During the solving phase, ask for a list of tactics to the current
     * formula.
     */
    public static final int UI_SOLVE_GET_APPLICABLE_TACTICS = 131;

    /**
     * During the solving phase, set the tactic to be applied to the current
     * formula.
     */
    public static final int UI_SOLVE_SET_NEXT_TACTIC = 132;

    /**
     * When entering a formula during the solving phase, request help on
     * entering the formula.
     */
    public static final int UI_SOLVE_HELP_ENTERING_FORMULA = 133;

    /**
     * During the solving phase, show which assumptions hold at the current
     * point of calculation.
     */
    public static final int UI_SOLVE_SHOW_ASSUMPTIONS = 140;

    /**
     * Show which tactic has been applied to get this formula
     */
    public static final int UI_SOLVE_TACTIC_APPLIED = 141;

    /**
     * show CalcHeadPanel; if CalcHead "complete" no interactiveElements if not,
     * startSpecifying
     */
    public static final int UI_SOLVE_SHOW_CALCHEAD = 142;

    /**
     * During the solving phase, request more detail about the currently
     * referenced element.
     */
    public static final int UI_SOLVE_SHOW_INTERMEDIATE_STEPS = 151;

    public static final int UI_DUMMY_LAST_SOLVE = 151;

    /** ***************** ACTIVE REQUESTS TO THE UI ********************** */

    /**
     * Request editing the currently active formula. DG distinguishes between
     * SPECIFY and SOLVE; thus could drop UI_SOLVE_EDIT_ACTIVE_FORMULA : int
     * 
     * LK050711 start editing implicit somewhere in tree, thus unused
     */
    public static final int UI_DO_EDIT_FORMULA = 229;

    /**
     * Request appending a new, empty formula.
     */
    public static final int UI_DO_APPEND_FORMULA = 230;

    /**
     * Activate an UI element.
     */
    public static final int UI_DO_ACTIVATE = 231;

    /**
     * Deactivate an UI element.
     */
    public static final int UI_DO_DEACTIVATE = 232;

    /**
     * The Dialog Guide has detached the listener and will not send any more
     * control requests. If registered as a Data Change Listener, update
     * notifications will still arrive!
     */
    public static final int UI_DO_DETACH = 233;

    /**
     * to transmit the problem from the problem browser to the worksheet
     */
    public static final int UI_PROBLEMBROWSER_PROBLEM_TO_WORKSHEET = 234;

    /**
     * to turn the auto match mode on
     */
    public static final int UI_PROBLEMBROWSER_AUTO_MATCH_ON = 235;

    /**
     * to turn the auto match mode off
     */
    public static final int UI_PROBLEMBROWSER_AUTO_MATCH_OFF = 236;

    /**
     * to open the problem browser window
     */
    public static final int UI_PROBLEMBROWSER_OPEN = 237;

    /**
     * to tell the proble browser dialog that the selection in the problem
     * browser has changed
     */
    public static final int UI_PROBLEMBROWSER_SELECTION_CHANGED = 238;

    /**
     * search for a more appropriate problem
     */
    public static final int UI_PROBLEMBROWSER_REFINE = 239;

    /**
     * no 'UserAction', but but included here for simplicity reasons: to send a
     * request fot the selected problem
     */
    public static final int UI_PROBLEMBROWSER_SELECTED_ID = 240;

    /**
     * no 'UserAction', but but included here for simplicity reasons: to send a
     * request fot the active worksheet
     */
    public static final int UI_PROBLEMBROWSER_ACTIVE_WORKSHEET = 241;

}