/*
 * Created on Oct 2, 2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package isac.wsdialog;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Vector;

import isac.bridge.CalcIterator;
import isac.interfaces.ICalcElement;
import isac.interfaces.ICalcIterator;
import isac.util.CalcChangedEvent;
import isac.util.formulae.Assumptions;
import isac.util.formulae.CalcHead;
import isac.util.formulae.CalcFormula;
import isac.util.formulae.Formula;
import isac.util.formulae.KEStoreID;
import isac.util.formulae.Match;
import isac.util.formulae.Position;
import isac.util.formulae.ProblemID;
import isac.util.tactics.Tactic;

/**
 * @author Alan Krempler
 * 
 * <code>ICalcIterator</code> on a calculation for use by the
 * <code>Worksheet</code>. Such an <code>ICalcIterator</code> is always
 * generated directly from a <code>CalcIterator</code>.
 */
//LK050511 this old decl. caused StubNotFoundException:
//public class DialogIterator extends UnicastRemoteObject
// implements ICalcIterator {
public class DialogIterator implements ICalcIterator, Serializable {

    /**
     * @param calcTree
     */
    protected CalcIterator calc_iterator_;

    protected IWorksheetDialog dialog_;

    /**
     * For use by the <code>Worksheet</code> the <code>WorksheetDialog</code>
     * replaces any <code>CalcIterator</code> on an <code>IToCalc</code> by
     * a <code>DialogIterator</code>
     */
    public DialogIterator(CalcIterator calc_iterator, IWorksheetDialog dialog)
            throws RemoteException {
        calc_iterator_ = calc_iterator;
        dialog_ = dialog;
    }

    /**
     * get the Position of the current element in the CalcTree
     */
    public Position getPosition() {
        return calc_iterator_.getPosition();
    }

    /**
     * Reference the first (root) element in the attached calculation tree.
     * 
     * @return Like all the moveXXX methods this returns true on success, false
     *         if there is no element to move to.
     */
    public boolean moveRoot() throws RemoteException {
        return calc_iterator_.moveRoot();
    }

    /**
     * Reference the first (root) element in the attached calculation tree.
     * 
     * @return Like all the moveXXX methods this returns true on success, false
     *         if there is no element to move to.
     */
    public boolean moveCalcHead() throws RemoteException {
        return calc_iterator_.moveCalcHead();
    }

    /**
     * Reference the previous formula in the attached calculation tree.
     * 
     * @return Like all the moveXXX methods this returns true on success, false
     *         if there is no element to move to.
     */
    public boolean moveUp() throws RemoteException {
        return calc_iterator_.moveUp();
    }

    /**
     * Reference the next formula in the attached calculation tree.
     * 
     * @return Like all the moveXXX methods this returns true on success, false
     *         if there is no element to move to.
     */
    public boolean moveDown() throws RemoteException {
        return calc_iterator_.moveDown();
    }

    /**
     * Enter the next coarser level of refinement in the attached calculation
     * tree.
     * 
     * @return Like all the moveXXX methods this returns true on success, false
     *         if there is no element to move to.
     */
    public boolean moveLevelUp() throws RemoteException {
        return calc_iterator_.moveLevelUp();
    }

    /**
     * Enter the next more detailed level of refinement in the attached
     * calculation tree.
     * 
     * @return Like all the moveXXX methods this returns true on success, false
     *         if there is no element to move to.
     */
    public boolean moveLevelDown() throws RemoteException {
        return calc_iterator_.moveLevelDown();
    }

    /**
     * Reference the tactic which resulted in the currently selected element of
     * the attached calculation tree.
     * <p>
     * A special case arises if the tactic to continue the current calculation
     * is solving a subproblem. The subproblem is a calculation on its own,
     * represented by a separate calculation(sub-)tree object. A reference to
     * the calculationtree representing the subproblem is passed to the
     * worksheet. The worksheet will handle the situation accordingly, e.g. by
     * opening a new worksheet or displaying the new calculationtree (indented)
     * in the current worksheet.
     * 
     * @return Like all the moveXXX methods this returns true on success, false
     *         if there is no element to move to.
     */
    public boolean moveTactic() throws RemoteException {

        return calc_iterator_.moveTactic();
    }

    /**
     * Reference the formula which resulted in the currently selected element of
     * the attached calculation tree.
     *  
     */
    public boolean moveFormula() throws RemoteException {
        return calc_iterator_.moveFormula();
    }

    /**
     * @see isac.interfaces.ICalcIterator#isLast()
     * @deprecated
     */
    public boolean isLast() throws RemoteException {
        return calc_iterator_.isLast();
    }

    /**
     * Use this to get the current element and check the type by calling
     * getType().
     * 
     * @return The currently referenced Element
     * @see getType()
     */
    public ICalcElement getElement() throws RemoteException {
        return calc_iterator_.getElement();
    }

    /**
     * The current level of nesting into subcalculations.
     * 
     * @return An int representing level of nesting (higher == more detailed)
     *         <p>
     *         Special values:
     *         <p>
     *         0: outmost level: only first and last step of calculation are on
     *         this level
     */
    public int getLevel() throws RemoteException {
        return calc_iterator_.getLevel();
    }

    public Object clone() {
        try {//just necessary for clone
            return new DialogIterator((CalcIterator) calc_iterator_.clone(),
                    dialog_);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public int compareTo(Object c) {
        if (c instanceof CalcIterator) {
            return calc_iterator_.compareTo(c);
        } else if (c instanceof DialogIterator) {
            return calc_iterator_
                    .compareTo(((DialogIterator) c).calc_iterator_);
        }
        throw new ClassCastException();
    }

    public boolean equals(Object o) {
        return (this.compareTo(o) == 0);
    }

    //WN050729 a hack left from following a wrong path
    // searching an error 
    //    public int hashCode() {
    //        return calc_iterator_.getPosition().toSMLString().hashCode();
    //    }

    /**
     * get a ICalcElement; if this is a CalcHead, start specifying (which
     * regards Usersettings)
     * 
     * this location in the code is closest to opening the CalcHeadPane which
     * corresponds to ...
     * 
     * @see isac.wsdialog.IWorksheetDialog#DIALOGPHASE_SPECIFY
     * @see isac.interfaces.ICalcIterator#getFormula()
     * @deprecated in favour of getElement()
     */
    public ICalcElement getFormula() throws RemoteException {
        ICalcElement fch = calc_iterator_.getFormula();
        //WN050512 outcommented until calls of getFormula are in control of
        // isac;
        //presently called automatically TOO OFTEN by Java renderinig
        //		if (fch instanceof CalcHead)
        //			fch = dialog_.startSpecifying();
        return fch;
    }

    /**
     * hand over elements from the CalcTree and filter them wrt their level
     * 
     * @see isac.interfaces.ICalcIterator#getElementsFromTo(ICalcIterator,
     *      ICalcIterator, Integer, boolean)
     */
    public Vector getElementsFromTo(ICalcIterator to, Integer level,
            boolean tacs_too) {
        Vector elems = null;
        try {
            elems = calc_iterator_.getElementsFromTo(to, level, tacs_too);
            //TODO.WN050626 filter all but last, if the level of 'from' = level
            // of 'to'
            // filter all levels which are lower than max of level 'from' or
            // 'to'
        } catch (RemoteException e) {
            e.printStackTrace();
        }
        return elems;
    }

    /**
     * @see isac.interfaces.ICalcIterator#getTactic()
     */
    public Tactic getTactic() throws RemoteException {
        return calc_iterator_.getTactic();
    }

    /**
     * @see isac.interfaces.ICalcIterator#getAssumption()
     */
    public Assumptions getAssumptions() throws RemoteException {
        return calc_iterator_.getAssumptions();
    }

    public String toSMLString() {
        return calc_iterator_.toSMLString();
    }

    /**
     * @see isac.interfaces.ICalcIterator#getApplicableTactics()
     */
    public Vector getApplicableTactics(int scope) throws RemoteException {
        return calc_iterator_.getApplicableTactics(scope);
    }

    /**
     * @see isac.interfaces.ICalcIterator#getAccumulatedAssumptions()
     */
    public Assumptions getAccumulatedAssumptions() throws RemoteException {
        return calc_iterator_.getAccumulatedAssumptions();
    }

    /**
     * @see isac.interfaces.ICalcIterator#onCalcHead()
     */
    public boolean onCalcHead() throws RemoteException {
        return calc_iterator_.onCalcHead();
    }

    /**
     * @see isac.interfaces.ICalcIterator#switchToGuard()
     */
    public boolean switchToGuard() throws RemoteException {
       return calc_iterator_.switchToGuard();
    }

    /**
     * @see isac.interfaces.ICalcIterator#switchToModel()
     */
    public boolean switchToModel() throws RemoteException {
        return calc_iterator_.switchToModel();
    }

    /**
     * @see isac.interfaces.ICalcIterator#tryMatchProblem(KEStoreID)
     */
    public Match tryMatchProblem(ProblemID problemID) throws RemoteException {
        return null;
    }

    /**
     * @see isac.interfaces.ICalcIterator#tryRefineProblem(KEStoreID)
     */
    public Match tryRefineProblem(ProblemID problemID) throws RemoteException {
        return null;
    }

    /**
     * @see isac.interfaces.ICalcIterator#tryMatchMethod(KEStoreID)
     */
    public Match tryMatchMethod(KEStoreID problemID) throws RemoteException {
        return null;
    }

}